# Copyright (C) 2012  Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import pywbem
from fmci_client_base import FmciBaseClient

class FmciPowerClient(FmciBaseClient):
    POWER_CLASS_NAME = "LMI_PowerManagementService"
    POWER_STATE_SUSPEND = 4
    POWER_STATE_FORCE_REBOOT = 5
    POWER_STATE_HIBERNATE = 7
    POWER_STATE_FORCE_POWEROFF = 8
    POWER_STATE_POWEROFF = 12
    POWER_STATE_REBOOT = 15

    def __init__(self, hostname, username = "", password = ""):
        super(self.__class__, self).__init__(hostname, username, password)

    def __powerCallMethod(self, method, **params):
        (inst, rparam) = self._getInstances(FmciPowerClient.POWER_CLASS_NAME)
        if not inst:
            return (False, rparam)
        (rval, _, errorstr) = self._callMethod(inst, method, **params)
        rval = rval in (0, 4096)
        return (rval, "" if rval else errorstr)

    def poweroff(self):
        return self.__powerCallMethod("RequestPowerStateChange",
            PowerState=pywbem.Uint16(FmciPowerClient.POWER_STATE_POWEROFF))

    def reboot(self):
        return self.__powerCallMethod("RequestPowerStateChange",
            PowerState=pywbem.Uint16(FmciPowerClient.POWER_STATE_REBOOT))

    def suspend(self):
        return self.__powerCallMethod("RequestPowerStateChange",
            PowerState=pywbem.Uint16(FmciPowerClient.POWER_STATE_SUSPEND))

    def hibernate(self):
        return self.__powerCallMethod("RequestPowerStateChange",
            PowerState=pywbem.Uint16(FmciPowerClient.POWER_STATE_HIBERNATE))
