# Copyright (C) 2012  Peter Hatina <phatina@redhat.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import pywbem
from fmci_client_base import FmciBaseClient

class FmciService(object):
    def __init__(self, client, inst):
        self.m_client = client
        self.m_instance = inst

    def start(self):
        return self.m_client.serviceStart(self.m_instance)

    def stop(self):
        return self.m_client.serviceStop(self.m_instance)

    def restart(self):
        return self.m_client.serviceRestart(self.m_instance)

    def enable(self):
        return self.m_client.serviceEnable(self.m_instance)

    def disable(self):
        return self.m_client.serviceDisable(self.m_instance)

    def reload(self):
        return self.m_client.serviceReload(self.m_instance)

    def tryRestart(self):
        return self.m_client.serviceTryRestart(self.m_instance)

    def condRestart(self):
        return self.m_client.serviceCondRestart(self.m_instance)

    def reloadRestart(self):
        return self.m_client.serviceReloadRestart(self.m_instance)

    def reloadTryRestart(self):
        return self.m_client.serviceReloadTryRestart(self.m_instance)

    def status(self):
        return self.m_client.serviceStatus(self.m_instance)

class FmciServiceClient(FmciBaseClient):
    SERVICE_CLASS_NAME = "LMI_Service"

    def __init__(self, hostname, username = "", password = ""):
        super(self.__class__, self).__init__(hostname, username, password)

    def __serviceGetInstance(self, service):
        (inst, errorstr) = self._getInstances(
            FmciServiceClient.SERVICE_CLASS_NAME, service,
            ErrorMsg = "No such service '" + service + "' found")
        if not inst:
            return (None, errorstr)
        return (inst, "")

    def __serviceCallMethod(self, service, method):
        (inst, errorstr) = self.__serviceGetInstance(service) if type(service) == str else (service, "")
        if not inst:
            return (False, errorstr)
        (rval, _, errorstr) = self._callMethod(inst, method)
        return (rval, errorstr)

    def __serviceGetProperty(self, service, prop):
        (inst, errorstr) = self.__serviceGetInstance(service) if type(service) == str else (service, "")
        if not inst or not prop in inst:
            return (False, errorstr)
        return (True, inst[prop])

    def serviceFind(self, service):
        (inst, errorstr) = self.__serviceGetInstance(service)
        if not inst:
            return (None, errorstr)
        return (FmciService(self, inst), "")

    def serviceStart(self, service):
        return self.__serviceCallMethod(service, "StartService")

    def serviceStop(self, service):
        return self.__serviceCallMethod(service, "StopService")

    def serviceRestart(self, service):
        return self.__serviceCallMethod(service, "RestartService")

    def serviceEnable(self, service):
        return self.__serviceCallMethod(service, "TurnServiceOn")

    def serviceDisable(self, service):
        return self.__serviceCallMethod(service, "TurnServiceOff")

    def serviceReload(self, service):
        return self.__serviceCallMethod(service, "ReloadService")

    def serviceTryRestart(self, service):
        return self.__serviceCallMethod(service, "TryRestartService")

    def serviceCondRestart(self, service):
        return self.__serviceCallMethod(service, "CondRestartService")

    def serviceReloadRestart(self, service):
        return self.__serviceCallMethod(service, "ReloadOrRestartService")

    def serviceReloadTryRestart(self, service):
        return self.__serviceCallMethod(service, "ReloadOrTryRestartService")

    def serviceStatus(self, service):
        return self.__serviceGetProperty(service, "Status")
